@@ -5,7 +5,6 @@ Validation should be done on a protocol level, i.e. not with gateway state.
 
 from __future__ import annotations
 
-from collections.abc import Mapping
 from dataclasses import dataclass
 from typing import TYPE_CHECKING, Literal, Self
 
@@ -50,22 +49,26 @@ class Message(DataClassDictMixin):
                 f"Failed to parse string {value} to message: {err}"
             ) from err
 
-        node_id = validation_schema.validate_node_id(data=data)
-        child_id = validation_schema.validate_child_id(data=data)
-        command = validation_schema.validate_command(data=data)
-        ack = validation_schema.validate_ack(data=data)
-        message_type = validation_schema.validate_message_type(data=data)
+        node_id = validation_schema.validate_node_id(raw_node_id=data["node_id"])
+        child_id, command, message_type = validation_schema.validate_child_id(
+            raw_child_id=data["child_id"],
+            raw_command=data["command"],
+            raw_message_type=data["message_type"],
+        )
+        ack = validation_schema.validate_ack(raw_ack=data["ack"])
         payload = data["payload"]
         return cls(node_id, child_id, command, ack, message_type, payload)
 
     def to_string(self, validation_schema: MessageSchema) -> str:
         """Serialize the message."""
         data = self.to_dict()
-        validation_schema.validate_node_id(data=data)
-        validation_schema.validate_child_id(data=data)
-        validation_schema.validate_command(data=data)
-        validation_schema.validate_ack(data=data)
-        validation_schema.validate_message_type(data=data)
+        validation_schema.validate_node_id(raw_node_id=data["node_id"])
+        validation_schema.validate_child_id(
+            raw_child_id=data["child_id"],
+            raw_command=data["command"],
+            raw_message_type=data["message_type"],
+        )
+        validation_schema.validate_ack(raw_ack=data["ack"])
         return (
             f"{self.node_id};{self.child_id};{self.command};"
             f"{self.ack};{self.message_type};{self.payload}\n"
@@ -78,11 +81,10 @@ class MessageSchema:
 
     protocol: ProtocolType
 
-    def validate_node_id(self, *, data: Mapping[str, str]) -> int:
+    def validate_node_id(self, *, raw_node_id: str) -> int:
         """Validate the node id field."""
-        value = data["node_id"]
         try:
-            node_id = int(value)
+            node_id = int(raw_node_id)
         except ValueError as err:
             raise InvalidMessageError("The node_id type must be an integer.") from err
 
@@ -96,14 +98,13 @@ class MessageSchema:
     def validate_child_id(
         self,
         *,
-        data: Mapping[str, str],
-        command: int | None = None,
-        message_type: int | None = None,
-    ) -> int:
+        raw_child_id: str,
+        raw_command: str,
+        raw_message_type: str,
+    ) -> tuple[int, int, int]:
         """Validate the child id field."""
-        value = data["child_id"]
         try:
-            child_id = int(value)
+            child_id = int(raw_child_id)
         except ValueError as exc:
             raise InvalidMessageError("The child_id type must be an integer.") from exc
 
@@ -112,16 +113,17 @@ class MessageSchema:
                 f"The child_id must be between 0 and {SYSTEM_CHILD_ID}, got {child_id}."
             )
 
-        if command is None:
-            command = self.validate_command(data=data, child_id=child_id)
-        if message_type is None:
-            message_type = self.validate_message_type(data=data, command=command)
+        command = self.validate_command(raw_command=raw_command, child_id=child_id)
+        message_type = self.validate_message_type(
+            raw_message_type=raw_message_type,
+            command=command,
+        )
 
         if (
             command == self.protocol.INTERNAL_COMMAND_TYPE
             and message_type in self.protocol.NODE_ID_REQUEST_TYPES
         ):
-            return child_id
+            return child_id, command, message_type
 
         if command in self.protocol.STRICT_SYSTEM_COMMAND_TYPES:
             valid_child_id = SYSTEM_CHILD_ID
@@ -132,29 +134,18 @@ class MessageSchema:
             if child_id != valid_child_id:
                 raise InvalidMessageError(error)
 
-        return child_id
+        return child_id, command, message_type
 
-    def validate_command(
-        self,
-        *,
-        data: Mapping[str, str],
-        child_id: int | None = None,
-        message_type: int | None = None,
-    ) -> int:
+    def validate_command(self, *, raw_command: str, child_id: int) -> int:
         """Validate a command."""
-        value = data["command"]
         try:
-            command = int(value)
+            command = int(raw_command)
         except ValueError as exc:
             raise InvalidMessageError("The command type must be an integer.") from exc
 
         command_type = self.protocol.Command
 
         valid_commands = {member.value for member in tuple(command_type)}
-        if child_id is None:
-            child_id = self.validate_child_id(
-                data=data, command=command, message_type=message_type
-            )
         if child_id == SYSTEM_CHILD_ID:
             valid_commands = self.protocol.VALID_SYSTEM_COMMAND_TYPES
 
@@ -166,32 +157,23 @@ class MessageSchema:
 
         return command
 
-    def validate_ack(self, *, data: Mapping[str, str]) -> Literal[0, 1]:
+    def validate_ack(self, *, raw_ack: str) -> Literal[0, 1]:
         """Validate an ack."""
-        value = data["ack"]
         try:
-            ack = int(value)
+            ack = int(raw_ack)
         except ValueError as exc:
             raise InvalidMessageError("The ack type must be an integer.") from exc
         if ack not in (0, 1):
             raise InvalidMessageError(f"The ack must be either 0 or 1, got {ack}.")
         return 1 if ack else 0
 
-    def validate_message_type(
-        self,
-        *,
-        data: Mapping[str, str],
-        command: int | None = None,
-    ) -> int:
+    def validate_message_type(self, *, raw_message_type: str, command: int) -> int:
         """Validate a message type."""
-        value = data["message_type"]
         try:
-            message_type = int(value)
+            message_type = int(raw_message_type)
         except ValueError as exc:
             raise InvalidMessageError("The message type must be an integer.") from exc
 
-        if command is None:
-            command = self.validate_command(data=data, message_type=message_type)
         valid_message_types = {
             member.value for member in self.protocol.VALID_COMMAND_TYPES[command]
         }
