@@ -1,45 +1,50 @@
 """Provide the protocol for MySensors version 1.4."""
 
+from __future__ import annotations
+
 import calendar
 from collections.abc import Awaitable, Callable, Coroutine
 from enum import IntEnum
 import time
-from typing import Any
+from typing import TYPE_CHECKING, Any
 
 from aiomysensors.exceptions import (
     MissingChildError,
     MissingNodeError,
     TooManyNodesError,
-    UnsupportedMessageError,
 )
-from aiomysensors.gateway import Gateway, MessageBuffer
-from aiomysensors.model.message import Message
-from aiomysensors.model.node import Node
-
-from . import (
+from aiomysensors.model.const import (
     DEFAULT_PROTOCOL_VERSION,
     MAX_NODE_ID,
     SYSTEM_CHILD_ID,
-    IncomingMessageHandlerBase,
 )
+from aiomysensors.model.message import Message
+from aiomysensors.model.node import Node
+
+from .message_handler import IncomingMessageHandlerBase
+
+if TYPE_CHECKING:
+    from aiomysensors.gateway import Gateway, MessageBuffer
+
+VERSION = "1.4"
 
 
 def handle_missing_protocol_version[
-    _IncomingMessageHandlerT: type[IncomingMessageHandler],
-    **_P,
+    IncomingMessageHandlerT: type[IncomingMessageHandler],
+    **P,
 ](
     func: Callable[
-        [_IncomingMessageHandlerT, Gateway, Message, MessageBuffer],
+        [IncomingMessageHandlerT, Gateway, Message, MessageBuffer],
         Awaitable[Message],
     ],
 ) -> Callable[
-    [_IncomingMessageHandlerT, Gateway, Message, MessageBuffer],
+    [IncomingMessageHandlerT, Gateway, Message, MessageBuffer],
     Coroutine[Any, Any, Message],
 ]:
     """Handle a missing set protocol version."""
 
     async def wrapper(
-        self: _IncomingMessageHandlerT,
+        self: IncomingMessageHandlerT,
         gateway: Gateway,
         message: Message,
         message_buffer: MessageBuffer,
@@ -98,7 +103,11 @@ class IncomingMessageHandler(IncomingMessageHandlerBase):
         """Process a presentation message."""
         if message.child_id == SYSTEM_CHILD_ID:
             # this is a presentation of a node
-            node = Node(message.node_id, message.message_type, message.payload)
+            node = Node(
+                node_id=message.node_id,
+                node_type=message.message_type,
+                protocol_version=message.payload,
+            )
             gateway.nodes[node.node_id] = node
             if message.node_id == 0:
                 # Set the gateway protocol version.
@@ -195,11 +204,7 @@ class IncomingMessageHandler(IncomingMessageHandlerBase):
         message_buffer: MessageBuffer,
     ) -> Message:
         """Process an internal message."""
-        try:
-            internal = gateway.protocol.Internal(message.message_type)
-        except ValueError as err:
-            raise UnsupportedMessageError(message, gateway.protocol_version) from err
-
+        internal = gateway.protocol.Internal(message.message_type)
         message_handler = getattr(cls, f"handle_{internal.name.lower()}", None)
 
         return await cls._handle_message(
@@ -221,11 +226,7 @@ class IncomingMessageHandler(IncomingMessageHandlerBase):
         if message.node_id not in gateway.nodes:
             raise MissingNodeError(message.node_id)
 
-        try:
-            stream = gateway.protocol.Stream(message.message_type)
-        except ValueError as err:
-            raise UnsupportedMessageError(message, gateway.protocol_version) from err
-
+        stream = gateway.protocol.Stream(message.message_type)
         message_handler = getattr(cls, f"handle_{stream.name.lower()}", None)
 
         return await cls._handle_message(
@@ -261,9 +262,9 @@ class IncomingMessageHandler(IncomingMessageHandlerBase):
 
         # Use temporary default values for the node until node sends presentation.
         gateway.nodes[next_id] = Node(
-            next_id,
-            Presentation.S_ARDUINO_NODE,
-            DEFAULT_PROTOCOL_VERSION,
+            node_id=next_id,
+            node_type=Presentation.S_ARDUINO_NODE,
+            protocol_version=DEFAULT_PROTOCOL_VERSION,
         )
         id_response_message = Message(
             node_id=message.node_id,
@@ -555,12 +556,12 @@ VALID_SYSTEM_COMMAND_TYPES = {
     Command.stream.value,
 }
 
-VALID_COMMAND_TYPES = {
-    Command.presentation: list(Presentation),
-    Command.set: list(SetReq),
-    Command.req: list(SetReq),
-    Command.internal: list(Internal),
-    Command.stream: list(Stream),
+VALID_COMMAND_TYPES: dict[int, list[IntEnum]] = {
+    Command.presentation.value: list(Presentation),
+    Command.set.value: list(SetReq),
+    Command.req.value: list(SetReq),
+    Command.internal.value: list(Internal),
+    Command.stream.value: list(Stream),
 }
 
 VALID_MESSAGE_TYPES = {
